@@ -68,10 +68,6 @@ test_that("flexdog works", {
   #                 ploidy = ploidy, model = "f1ppdr", verbose = FALSE)
   # )
 
-  fout <- flexdog(refvec = refvec, sizevec = sizevec,
-                  p1ref = 10, p1size = 20,
-                  ploidy = ploidy, model = "s1pp", verbose = FALSE)
-
   flexdog(refvec = refvec,
           sizevec = sizevec,
           ploidy = ploidy,
@@ -82,73 +78,6 @@ test_that("flexdog works", {
 }
 )
 
-test_that("f1ppdr works on SNP3", {
-  skip("f1ppdr not quite yet")
-  data("snpdat")
-  fout <- flexdog_full(refvec  = snpdat$counts[snpdat$snp == "SNP3"],
-                       sizevec = snpdat$size[snpdat$snp == "SNP3"],
-                       ploidy  = 6,
-                       model   = "f1ppdr",
-                       verbose = FALSE, bias = 2)
-
-  fout <- flexdog(refvec  = snpdat$counts[snpdat$snp == "SNP1"],
-                       sizevec = snpdat$size[snpdat$snp == "SNP1"],
-                       ploidy  = 6,
-                       model   = "f1ppdr",
-                       verbose = FALSE)
-
-  fout <- flexdog(refvec  = snpdat$counts[snpdat$snp == "SNP3"],
-                  sizevec = snpdat$size[snpdat$snp == "SNP3"],
-                  ploidy  = 6,
-                  model   = "s1pp",
-                  verbose = FALSE)
-})
-
-test_that("get_uni_rep is inverse of get_probk_vec", {
-  probvec <- dbinom(0:6, 6, 0.4)
-  pout <- get_uni_rep(probvec)
-  probvec2 <- get_probk_vec(pivec = pout$pivec,
-                            model = "ash",
-                            mode = pout$mode - 1)
-  expect_equal(probvec, probvec2)
-
-  probvec <- dbinom(0:6, 6, 0.9)
-  pout <- get_uni_rep(probvec)
-  probvec2 <- get_probk_vec(pivec = pout$pivec,
-                            model = "ash",
-                            mode = pout$mode - 1)
-  expect_equal(probvec, probvec2)
-})
-
-test_that("get right mode in initialization", {
-  ploidyvec <- 2:30
-  for (ploidy in ploidyvec) {
-    modevec <- 0:(ploidy - 1) + 0.5
-    for (index in 1:length(modevec)) {
-      mode <- modevec[index]
-      pvec_init <- stats::dbinom(x = 0:ploidy, size = ploidy,
-                                 prob = floor(mode) / ploidy)
-      pout <- get_uni_rep(pvec_init)
-      expect_equal(pout$mode - 1, mode)
-    }
-  }
-})
-
-test_that("get_probk_vec works", {
-  pivec <- c(0.3, 0.2, 0.5)
-
-  expect_equal(get_probk_vec(pivec = pivec, model = "flex", mode = -1), pivec)
-
-  expect_equal(get_probk_vec(pivec = pivec, model = "ash", mode = 2.5), c(0.1, 0.2, 0.7))
-
-  expect_equal(get_probk_vec(pivec = pivec, model = "ash", mode = 1.5), c(0.15, 0.35, 0.5))
-
-  expect_equal(get_probk_vec(pivec = pivec, model = "ash", mode = 0.5), c(0.3, 0.45, 0.25))
-
-  expect_equal(get_probk_vec(pivec = pivec, model = "ash", mode = -0.5), c(0.4 + 0.5/3, 0.1 + 0.5/3, 0.5/3))
-}
-)
-
 test_that("don't update bias, seq, od when supposed not to", {
   refvec  <- 1:20
   sizevec <- 40:21
@@ -245,42 +174,24 @@ test_that("actually using parent data", {
   expect_equal(fout3$par$pgeno, 3)
 })
 
+test_that("genotype likelihoods and posteriors are consistent", {
+  refvec <- c(1, 2, 1, 3, 2)
+  sizevec <- c(4, 2, 2, 3, 4)
+  ploidy <- 4
 
-test_that("get_wik_mat_out has expected behavior", {
-  ploidy    <- 4
-  probk_vec <- seq(1 / (ploidy + 1), ploidy + 1)
-  refvec    <- 1:5
-  sizevec   <- 6:10
-  seq       <- 0.001
-  bias      <- 0.25
-  od        <- 0.02
-  wikmat <- get_wik_mat_out(probk_vec = probk_vec,
-                            out_prop  = 1,
-                            refvec    = refvec,
-                            sizevec   = sizevec,
-                            ploidy    = ploidy,
-                            seq       = seq,
-                            bias      = bias,
-                            od        = od)
-  expect_true(all(wikmat[1:5, 1:5] == 0))
-  expect_true(all(wikmat[, 6] == 1))
+  fout <- flexdog(refvec = refvec,
+                  sizevec = sizevec,
+                  ploidy = ploidy,
+                  verbose = FALSE)
 
-  wikmat1 <- get_wik_mat_out(probk_vec = probk_vec,
-                             out_prop  = 0,
-                             refvec    = refvec,
-                             sizevec   = sizevec,
-                             ploidy    = ploidy,
-                             seq       = seq,
-                             bias      = bias,
-                             od        = od)
-
-  wikmat2 <- get_wik_mat(probk_vec = probk_vec,
-                         refvec    = refvec,
-                         sizevec   = sizevec,
-                         ploidy    = ploidy,
-                         seq       = seq,
-                         bias      = bias,
-                         od        = od)
-
-  expect_equal(wikmat1[1:5, 1:5], wikmat2)
+  ind <- 2
+  probhand <- fout$gene_dist * exp(fout$genologlike[ind,])
+  probhand <- probhand / sum(probhand)
+  expect_equal(
+    fout$postmat[ind,],
+    probhand,
+    tolerance = 0.005
+  )
 })
+
+
